function [f] = DesignModel(xi,zz, a,ir,xi0)
% DESIGNMODEL is the ODE describing the change of states along the path
% 
% uu,xi0,dxi,nc,VRoF0)
%
% y =zz(1:nc)
% T =zz(nc+1)
% ga=zz(nc+2)
%
% uM=uu(1)-uu(2) linear
% uH=uu(3)
% uF=uu(4)
% uT=uu(5)
%
% R = [Ry; RT]
% Ry(1:nc) = mol/(m3*s)
% RT = mol*K/(m3*s)

% 
% Assign the design functions 
%
%xi0=sum(a.dxi(1:ir-1));
uM = a.uM(2*ir-1) + (xi-xi0)/a.dxi(ir)*(a.uM(2*ir)-a.uM(2*ir-1)); % For piecewise - linear 
uH = a.sigma*40*(a.uH(2*ir-1) + ...
     (xi-xi0)/a.dxi(ir)*(a.uH(2*ir) - a.uH(2*ir-1)));
uF = a.uF0(ir)*a.sigma*(a.uF(2*ir-1) + ...
     (xi-xi0)/a.dxi(ir)*(a.uF(2*ir) - a.uF(2*ir-1)));
uFF = a.uF0(ir)*a.sigma*(a.uFF(2*ir-1) + ...
      (xi-xi0)/a.dxi(ir)*(a.uFF(2*ir) - a.uFF(2*ir-1)));
uA = a.uA(2*ir-1) + (xi-xi0)/a.dxi(ir) * ( a.uA(2*ir)-a.uA(2*ir-1) );
%uS=a.uS(2*ir-1) + (xi-xi0)/a.dxi(ir) * ( a.uS(2*ir)-a.uS(2*ir-1) );
uT = a.uT(2*ir-1) + (xi-xi0)/a.dxi(ir) * ( a.uT(2*ir)-a.uT(2*ir-1) );

%uH = a.uH(ir) *a.sigma*40;    %kg/(s m3) a.uH(ir) is scaled 07-10-2009-mhi
%uH=uH*a.sigma*40;
% uF = a.uF0(ir)*a.sigma;	%kg/(s m3) If point feed uF0 has been set to 
                            %zero before integration and in
                            %ModelConstrain
% uFF= a.uFF0(ir)*a.sigma;
% uT = a.uT(ir);                % (Tw-Tref)/Tref
% uA = a.uA(ir);

xx = zz(1:a.nxx); 
ga = zz(a.nxx+1);               %ga=F/F0         %@mhi 050608
nc = a.nc;

cp   = a.cp*xx(1:nc);           %@mhi 120608
cpF  = a.cp*a.xxF(1:nc);        %@mhi 120608
cpFF = a.cp*a.xxFF(1:nc);
cpR  = a.cp*a.xx0(1:nc);
R    = feval(a.kinetics,xx,uA,cp,a.M',a.Tref,a.pTot);
Rt   = a.sigma*R;
Rt(1:nc)  = Rt(1:nc)  + uF*       (a.xxF(1:nc) - xx(1:nc));
Rt(1:nc)  = Rt(1:nc)  + uFF*      (a.xxFF(1:nc) - xx(1:nc));
Rt(a.nxx) = Rt(a.nxx) + uF*cpF/cp*(a.xxF(a.nxx) - xx(a.nxx)) + uFF*cpFF/cp*(a.xxFF(a.nxx) - xx(a.nxx));
Rt(a.nxx) = Rt(a.nxx) - uH*cpR/cp*(xx(a.nxx)-uT);
% 
%if uM>0,   
    J = Jacobian(a,xx,uA,cp,a.M');
    %J= Jgraaf2(xx,uA,cp,a.M',a.Tref);
    %if xi>0.5&xi<0.6, disp(xi); disp(Ja); disp(J); end
    Jt = a.sigma*J;
    Jt(a.nxx,a.nxx) = Jt(a.nxx,a.nxx) - uH*cpR/cp + uF*(1-cpF/cp) + uFF*(1-cpFF/cp);
    J  =(ga*eye(a.nxx)-uM*Jt);
    f  = J\Rt;
%else
%    f  = Rt/ga;
%end
dgadxi = uF + uFF;          % uF=sigma*alpha @mhi 050608
f = [f; dgadxi];