function [M, Cp, liqFrac] = uniSimPropertiesTodicLump(T,P)
% Function extracting the heat capacity for the components in comstring
% from FlowPropertiesTodicLump.usc (Unisim file) at temperature T and pressure P.
% compString = {'CO','CO2','H2','H2O','C1','C2','C3-4','C5-10','C11+'};
% Input:
%   T - Temperature [C]
%   P - Pressure [Pa]

% Output:
%   Cp - Heat capacity [kJ/(kg*K]]

% Assumed weight fraction in lumps:
%   Equal amounts (mole fraction) of each component

% Written by: Martin S. Foss, Spring 2013
%-------------------------------------------------------------------%
serv = actxserver('UniSimDesign.Application');  
usdCase = serv.ActiveDocument;

sheet = usdCase.Flowsheet;
stream = sheet.MaterialStreams;

compString = {'CO','CO2','H2','H2O','C1','C2','C3-4','C5-10','C11+'};
Cp = zeros(length(compString),1);
M  = zeros(length(compString),1);
liqFrac = zeros(length(compString),1);

for i = 1:length(compString)
    stream.Item(compString{i}).Pressure.Value = P/1000; %[kPa]
    stream.Item(compString{i}).Temperature.Value = T; %[C]
    
    Cp(i,1) = stream.Item(compString{i}).MassHeatCapacityValue;
    M(i,1) = stream.Item(compString{i}).MolecularWeightValue;
    liqFrac(i,1) = stream.Item(compString{i}).LiquidFractionValue;
end
return